# # encoding: utf-8

# from django import forms
# from django.utils.translation import ugettext_lazy as _

# from localflavor.fr.forms import FRPhoneNumberField as OriginalFRPhoneNumberField

# class FRPhoneNumberField(OriginalFRPhoneNumberField):
#     default_error_messages = {
#         'invalid': _(u"Le numéro de téléphone doit correspondre au format 0X XX XX XX XX."),
#     }